{% extends "base.html" %}
{% load i18n %}

{% block content %}

    <div class="age-gateway">
        <div class="title">{% trans "Confirm your age" %}</div>
            
        <form action="{% url age-gateway %}" method="POST" class="foundry-form">   
            {% csrf_token %}
            {{ form.as_div }}
            <div class="buttons">
                <input type="submit" value="{% trans "Confirm" %}" />
            </div>
        </form>

    </div>
{% endblock content %}
