{% load jmbo_template_tags foundry_tags %}

{% comment %}
Layout similar to a classic iPod.
{% endcomment %}

{% jmbocache 3600 'modelbase-list-item-ipod-inclusion' object.id object.modified %}
    <div class="image">
        <a href="{{ object.get_absolute_url }}">
            <img src="{{ object.get_modelbase_list_item_ipod_url }}" alt="{{ object.title }}" />
        </a>
    </div>

    <div class="title">
        {% if object.categories.all.exists %}<span class="category">{{ object.categories.all|join_titles }}:</span>{% endif %}
        <a href="{{ object.get_absolute_url }}">{{ object.title }}</a>
    </div>
{% endjmbocache %}

{% include "includes/list_meta.html" %}
