{% extends "base.html" %}
{% load foundry_tags %}

{% block extratitle %}{% if not object.is_homepage %} - {{ object.title }}{% endif %}{% endblock %}

{% block page_content %}
    {% if object.css %}
        <style type="text/css">
            {{ object.css|safe }}
        </style>
    {% endif %}
    {% with object.rows_by_block_name.content as rows %}
        {% include "foundry/inclusion_tags/rows.html" %}
    {% endwith %}
{% endblock %}
